package net.ipksindia

import dao.TellerDao
import net.ipksindia.dao.TransactionDao
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import response.TransactionFailureResponse
import response.TransactionSuccessResponse

class NeftRequestProcessor {

    companion object {
        private val logger: Logger = LoggerFactory.getLogger(NeftRequestProcessor::class.java)
        private val migratedDCCBCodes = listOf("0003","0015", "0016")
        val bankDccbToSftpMap = mutableMapOf<String, String>(
            "0015" to "0005", //Tamluk
            "0003" to "0021", //Balageria
            "0016" to "0001", //WBSCB
        )

        fun process(transactionNumber: String): Pair<String, String>? {

            val outwardTransaction = TransactionDao().getTransactionRequest(transactionNumber) ?: run {
                logger.error("TXN: #{} FAILED REASON: Transaction not found", transactionNumber)
                return null
            }
            logger.info("TXN: #{} FOUND", transactionNumber)

            val dccbCode = outwardTransaction.dccbCode.padStart(4, '0')
            val branchCode = outwardTransaction.branchCode.padStart(5, '0')

            if(dccbCode !in migratedDCCBCodes) {
                logger.error("TXN: #{} FAILED REASON: DCCB Code not migrated", transactionNumber)
                return null
            }


            val makerTeller = TellerDao.getTeller(dccbCode, branchCode) ?: run {
                logger.error("TXN: #{} FAILED REASON: Teller not found", transactionNumber)
                return null
            }
            val transactionPair = TransactionFactory(outwardTransaction, makerTeller).createTransactionPair()
            val (transferResponse, neftResponse) = try {
                TransactionExecutor().executePair(transactionPair)
            } catch (e: Exception) {
                logger.error("TXN: #{} FAILED REASON: {}", transactionNumber, e.message)
                return null
            }

            logger.info(
                "TXN: #{} TRF_TXN: {} NEFT_TXN: {}",
                transactionNumber,
                transferResponse.status,
                neftResponse.status
            )

            if (transferResponse.status == "SUCCESS" && neftResponse.status == "SUCCESS") {
                val transferQueueNumber = (transferResponse as TransactionSuccessResponse).queueNumber
                val neftQueueNumber = (neftResponse as TransactionSuccessResponse).queueNumber
                try {
                    TransactionDao().updateSuccessTransaction(outwardTransaction, transferQueueNumber, neftQueueNumber)
                    logger.info("TXN: #{} UPDATED RESULTS SUCCESSFULLY", transactionNumber)
                } catch (e: Exception) {
                    logger.error(
                        "TXN: #{} QUEUE INITIATED BUT FAILED TO UPDATE RESULT: {}",
                        transactionNumber,
                        e.message
                    )
                }
                return Pair(transferQueueNumber, neftQueueNumber)
            } else {
                val transferErrorMsg = (transferResponse as TransactionFailureResponse).errorMsg
                val neftErrorMsg = (neftResponse as TransactionFailureResponse).errorMsg
                logger.error(
                    "TXN: #{} TRANSFER TXN FAILED DUE TO: {}",
                    transactionNumber,
                    transferErrorMsg
                )
                logger.error(
                    "TXN: #{} NEFT TXN FAILED DUE TO: {}",
                    transactionNumber,
                    neftErrorMsg
                )
            }
            return null
        }
    }
}